' <auto-generated />

Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Microsoft.Xrm.Sdk
Imports Microsoft.Xrm.Sdk.Query
Imports System.Reflection
Imports Microsoft.Crm.Sdk.Messages
Imports Microsoft.Xrm.Sdk.Messages
Namespace YourCustomNameSpace
    Public MustInherit Class BaseProxyClass
        Protected Sub New(Original As Entity)
            If String.IsNullOrEmpty(Original.LogicalName) Then
                Throw New Exception("Please specify the 'LogicalName' on the entity when using a proxy class.")
            End If
            Dim EntityName As String = GetLogicalName(Me.GetType())
            If EntityName <> Original.LogicalName Then
                Throw New Exception("Please make sure that the entity logical name matches that of the proxy class you are creating.")
            End If
            _entity = Original
        End Sub
        Public Shared ReturnDatesInLocalTime As Boolean = False
        Private ReadOnly _entity As Entity
        Public ReadOnly Property Entity() As Entity
            Get
                Return _entity
            End Get
        End Property
        Public Property Id() As Guid
            Get
                Return Me.Entity.Id
            End Get
            Set(value As Guid)
                Me.Entity.Id = value
            End Set
        End Property
        Public Property LogicalName() As String
            Get
                Return Me.Entity.LogicalName
            End Get
            Set(value As String)
                Me.Entity.LogicalName = value
            End Set
        End Property
        Public Sub Create(Service As IOrganizationService)
            Dim EntityId As Guid = Service.Create(Me.Entity)
            Me.Entity.Id = EntityId
            mChangedValues.Clear()
        End Sub
        Public Sub Delete(Service As IOrganizationService)
            Service.Delete(Me.Entity.LogicalName, Me.Entity.Id)
        End Sub
        Public Sub Update(Service As IOrganizationService)
            Dim entity As New Entity(Me.LogicalName)
            entity.Id = Me.Id
            For Each attributeName As String In mChangedValues.Keys
                entity.Attributes(attributeName) = Me.Entity.Attributes(attributeName)
            Next
            Service.Update(entity)
            mChangedValues.Clear()
        End Sub
        Public Sub Save(Service As IOrganizationService)
            If Me.Entity.Id <> Guid.Empty Then
                Me.Update(Service)
            Else
                Me.Create(Service)
            End If
        End Sub
        Private Shared ProxyTypes As New Dictionary(Of Type, String)()
        Public Shared Sub RegsterProxyType(ProxyType As Type, LogicalName As String)
            If Not ProxyTypes.ContainsKey(ProxyType) Then
                ProxyTypes.Add(ProxyType, LogicalName)
            End If
        End Sub
        Public Shared Function GetLogicalName(Of T As BaseProxyClass)() As String
            Return GetLogicalName(GetType(T))
        End Function
        Public Shared Function GetLogicalName(type As Type) As String
            If Not ProxyTypes.ContainsKey(type) Then
                'This forces the static constructor to be called and register the name
                Dim InfoTemp As ConstructorInfo = type.GetConstructor(New Type() {})
                InfoTemp.Invoke(New Object() {})
            End If
            Return ProxyTypes(type)
        End Function
        Public Shared Widening Operator CType(Proxy As BaseProxyClass) As EntityReference
            If Proxy IsNot Nothing Then
                Return New EntityReference(Proxy.LogicalName, Proxy.Id)
            End If
            Return Nothing
        End Operator
        Public Shared Widening Operator CType(Proxy As BaseProxyClass) As Entity
            If Proxy IsNot Nothing Then
                Return Proxy.Entity
            End If
            Return Nothing
        End Operator
        Protected Function GetPropertyValue(Of T)(Name As String) As T
            If Me.Entity.Contains(Name) Then
                Dim Value As T = DirectCast(Me.Entity.Attributes(Name), T)
                If (GetType(T) Is GetType(DateTime) OrElse GetType(T) Is GetType(System.Nullable(Of DateTime))) AndAlso ReturnDatesInLocalTime Then
                    Value = DirectCast(DirectCast(DirectCast(DirectCast(Value, [Object]), DateTime).ToLocalTime(), [Object]), T)
                End If
                Return Value
            End If
            Return Nothing
        End Function
        'dictionary of values that have changed along with the original value
        Private mChangedValues As New Dictionary(Of String, Object)()
        Private mEqualityComparer As New AttributeEqualityComparer()
        Protected Sub SetPropertyValue(Of T)(Name As String, Value As T)
            If mChangedValues.ContainsKey(Name) Then
                Dim OriginalValue As Object = mChangedValues(Name)
                Dim CurrentValue As Object = If(Me.Entity.Contains(Name), DirectCast(Me.GetPropertyValue(Of T)(Name), Object), Nothing)
                If Not mEqualityComparer.Equals(CurrentValue, Value) Then
                    If mEqualityComparer.Equals(OriginalValue, Value) Then
                        mChangedValues.Remove(Name)
                    End If
                End If
            Else
                Dim CurrentValue As Object = If(Me.Entity.Contains(Name), DirectCast(Me.GetPropertyValue(Of T)(Name), Object), Nothing)
                If Not mEqualityComparer.Equals(CurrentValue, Value) Then
                    mChangedValues.Add(Name, CurrentValue)
                End If
            End If
            Me.Entity.Attributes(Name) = Value
        End Sub
        Public ReadOnly Property IsDirty() As Boolean
            Get
                Return Me.mChangedValues.Count > 0
            End Get
        End Property
        Public Shared Function GetRelatedOneToManyEntities(Service As IOrganizationService, RelatedId As Guid, LogicalName As String, AttributeName As String, ParamArray Columns As String()) As List(Of Entity)
            Dim ColumnSet As ColumnSet = If((Columns.Length = 0), New ColumnSet(True), New ColumnSet(Columns))
            Return GetRelatedOneToManyEntities(Service, RelatedId, LogicalName, AttributeName, ColumnSet)
        End Function
        Public Shared Function GetRelatedOneToManyEntities(Service As IOrganizationService, RelatedId As Guid, LogicalName As String, AttributeName As String, Columns As ColumnSet) As List(Of Entity)
            Return Service.RetrieveMultiple(LogicalName, New ConditionExpression(AttributeName, ConditionOperator.Equal, RelatedId), Columns).Entities.ToList()
        End Function
        Public Shared Function GetRelatedManyToManyEntities(Service As IOrganizationService, Proxy As BaseProxyClass, LinkFromAttributeName As String, RelatedEntityName As String, LinkToAttributeName As String, IntermediateTableName As String, ParamArray Columns As String()) As List(Of Entity)
            Dim ColumnSet As ColumnSet = If((Columns.Length = 0), New ColumnSet(True), New ColumnSet(Columns))
            Return GetRelatedManyToManyEntities(Service, Proxy, LinkFromAttributeName, RelatedEntityName, LinkToAttributeName, IntermediateTableName, ColumnSet)
        End Function
        Public Shared Function GetRelatedManyToManyEntities(Service As IOrganizationService, Proxy As BaseProxyClass, LinkFromAttributeName As String, RelatedEntityName As String, LinkToAttributeName As String, IntermediateEntityName As String, Columns As ColumnSet) As List(Of Entity)
            Dim Condition As New ConditionExpression() With { _
               .AttributeName = LinkFromAttributeName, _
               .[Operator] = ConditionOperator.Equal
            }
            Condition.Values.Add(Proxy.Id)

            Dim Link1 As New LinkEntity() With { _
                .LinkFromEntityName = RelatedEntityName, _
                .LinkFromAttributeName = LinkToAttributeName, _
                .LinkToEntityName = IntermediateEntityName, _
                .LinkToAttributeName = LinkToAttributeName
            }
            Link1.LinkCriteria.AddCondition(Condition)

            Dim Link2 As New LinkEntity() With { _
                        .LinkFromEntityName = IntermediateEntityName, _
                        .LinkFromAttributeName = LinkFromAttributeName, _
                        .LinkToEntityName = Proxy.LogicalName, _
                        .LinkToAttributeName = LinkFromAttributeName
                    }
            Link2.LinkCriteria.AddCondition(Condition)
            Link1.LinkEntities.Add(Link2)

            Dim Query As New QueryExpression() With { _
                .EntityName = RelatedEntityName, _
                .ColumnSet = Columns
            }
            Query.LinkEntities.Add(Link1)

            Return Service.RetrieveMultiple(Query).Entities.ToList()
        End Function
        Private Class AttributeEqualityComparer
            Implements IEqualityComparer
            Public Overloads Function Equals(x As Object, y As Object) As Boolean Implements IEqualityComparer.Equals
                If (x Is Nothing OrElse (x.[GetType]() Is GetType(String) AndAlso String.IsNullOrEmpty(TryCast(x, String)))) AndAlso (y Is Nothing OrElse (y.[GetType]() Is GetType(String) AndAlso String.IsNullOrEmpty(TryCast(y, String)))) Then
                    Return True
                Else
                    If x Is Nothing AndAlso y Is Nothing Then
                        Return True
                    ElseIf x Is Nothing AndAlso y IsNot Nothing Then
                        Return False
                    ElseIf x IsNot Nothing AndAlso y Is Nothing Then
                        Return False
                    ElseIf x.GetType() Is y.[GetType]() Then
                        If x.GetType() Is GetType(OptionSetValue) Then
                            Return DirectCast(x, OptionSetValue).Value = DirectCast(y, OptionSetValue).Value
                        ElseIf x.[GetType]() Is GetType(EntityReference) Then
                            If DirectCast(x, EntityReference).LogicalName = DirectCast(y, EntityReference).LogicalName Then
                                Return DirectCast(x, EntityReference).Id = DirectCast(y, EntityReference).Id
                            Else
                                Return False
                            End If
                        ElseIf x.GetType() Is GetType(Money) Then
                            Return (DirectCast(x, Money).Value = DirectCast(y, Money).Value)
                        ElseIf x.[GetType]() Is GetType(DateTime) OrElse x.[GetType]() Is GetType(System.Nullable(Of DateTime)) Then
                            'Compare only down to the second since CRM only tracks down to the second
                            Return Math.Abs((DirectCast(x, DateTime) - DirectCast(y, DateTime)).TotalSeconds) < 1
                        Else
                            Return x.Equals(y)
                        End If
                    Else
                        Return False
                    End If
                End If
            End Function
            Public Overloads Function GetHashCode(obj As Object) As Integer Implements IEqualityComparer.GetHashCode
                Return obj.GetHashCode()
            End Function
        End Class
    End Class

    Public Module ExtensionMethods
        <System.Runtime.CompilerServices.Extension> _
        Public Sub Create(Service As IOrganizationService, Proxy As BaseProxyClass)
            Dim EntityId As Guid = Service.Create(Proxy.Entity)
            Proxy.Id = EntityId
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub Update(Service As IOrganizationService, Proxy As BaseProxyClass)
            Proxy.Update(Service)
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub Delete(Service As IOrganizationService, Proxy As BaseProxyClass)
            Service.Delete(Proxy.LogicalName, Proxy.Id)
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub SetState(Service As IOrganizationService, Proxy As BaseProxyClass, State As Integer, Status As Integer)
            SetState(Service, Proxy, New OptionSetValue(State), New OptionSetValue(Status))
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub SetState(Service As IOrganizationService, Proxy As BaseProxyClass, State As OptionSetValue, Status As OptionSetValue)
            Dim Request As New SetStateRequest() With {.EntityMoniker = Proxy, .State = State, .Status = Status}
            Service.Execute(Request)
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Function ToProxy(Of T As BaseProxyClass)(entity As Entity) As T
            If entity IsNot Nothing Then
                Dim Constructor As ConstructorInfo = GetType(T).GetConstructor(New Type() {GetType(Entity)})
                Return entity.ToProxy(Of T)(Constructor)
            End If
            Return Nothing
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Private Function ToProxy(Of T As BaseProxyClass)(entity As Entity, construcor As ConstructorInfo) As T
            Return TryCast(construcor.Invoke(New Object() {entity}), T)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxies(Of T As BaseProxyClass)(Service As IOrganizationService, expression As ConditionExpression, columns As ColumnSet) As List(Of T)
            Dim FilterExpression As New FilterExpression()
            FilterExpression.AddCondition(expression)
            Return RetrieveProxies(Of T)(Service, FilterExpression, columns)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxies(Of T As BaseProxyClass)(Service As IOrganizationService, expression As FilterExpression, columns As ColumnSet) As List(Of T)
            Dim Query As New QueryExpression(BaseProxyClass.GetLogicalName(Of T)()) With {.ColumnSet = columns, .Criteria = expression}
            Return RetrieveProxies(Of T)(Service, Query)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxies(Of T As BaseProxyClass)(Service As IOrganizationService, query As QueryExpression) As List(Of T)
            Dim Entities As EntityCollection = Service.RetrieveMultipleAllPages(query)
            Return Entities.ToProxies(Of T)()
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function ToProxies(Of T As BaseProxyClass)(Entities As EntityCollection) As List(Of T)
            Return Entities.Entities.ToProxies(Of T)()
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function ToProxies(Of T As BaseProxyClass)(Entities As IEnumerable(Of Entity)) As List(Of T)
            Dim Constructor As ConstructorInfo = GetType(T).GetConstructor(New Type() {GetType(Entity)})
            Return (From EntityTemp In Entities Select EntityTemp.ToProxy(Of T)(Constructor)).ToList()
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxy(Of T As BaseProxyClass)(Entity As EntityReference, Service As IOrganizationService, ParamArray Columns As String()) As T
            Return Service.RetrieveProxy(Of T)(Entity.Id, Columns)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxy(Of T As BaseProxyClass)(Entity As EntityReference, Service As IOrganizationService, Columns As ColumnSet) As T
            Return Service.RetrieveProxy(Of T)(Entity.Id, Columns)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxy(Of T As BaseProxyClass)(Service As IOrganizationService, Id As Guid, ParamArray Columns As String()) As T
            Dim entityName As String = BaseProxyClass.GetLogicalName(Of T)()
            Return RetrieveEntity(New EntityReference(entityName, Id), Service, Columns).ToProxy(Of T)()
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveProxy(Of T As BaseProxyClass)(Service As IOrganizationService, Id As Guid, columns As ColumnSet) As T
            Dim entityName As String = BaseProxyClass.GetLogicalName(Of T)()
            Return RetrieveEntity(New EntityReference(entityName, Id), Service, columns).ToProxy(Of T)()
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveEntity(Entity As EntityReference, Service As IOrganizationService, ParamArray Columns As String()) As Entity
            Dim ColumnSet As New ColumnSet(Columns)
            If ColumnSet.Columns.Count = 0 Then
                ColumnSet = New ColumnSet(True)
            End If
            Return Entity.RetrieveEntity(Service, ColumnSet)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveEntity(Entity As EntityReference, Service As IOrganizationService, Columns As ColumnSet) As Entity
            If Columns Is Nothing Then
                Columns = New ColumnSet(True)
            End If
            Return Service.Retrieve(Entity.LogicalName, Entity.Id, Columns)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveMultiple(Service As IOrganizationService, LogicalName As String, Condition As ConditionExpression, ParamArray Columns As String()) As EntityCollection
            Dim ColumnSet As ColumnSet = If((Columns Is Nothing), New ColumnSet(True), New ColumnSet(Columns))
            Return RetrieveMultiple(Service, LogicalName, Condition, ColumnSet)
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveMultiple(Service As IOrganizationService, LogicalName As String, Condition As ConditionExpression, Columns As ColumnSet) As EntityCollection
            Dim Query As New QueryExpression(LogicalName)
            Query.Criteria.AddCondition(Condition)
            Query.ColumnSet = Columns
            Return Service.RetrieveMultipleAllPages(Query)
        End Function
        Public RetrieveAllPages As Boolean = True
        <System.Runtime.CompilerServices.Extension> _
        Public Function RetrieveMultipleAllPages(Service As IOrganizationService, Query As QueryExpression) As EntityCollection
            Dim Results As EntityCollection = Service.RetrieveMultiple(Query)
            While Results.MoreRecords AndAlso RetrieveAllPages
                Dim CurrentResults As IList(Of Entity) = Results.Entities
                Query.PageInfo.PagingCookie = Results.PagingCookie
                Query.PageInfo.PageNumber += 1
                Results = Service.RetrieveMultiple(Query)
                Results.Entities.AddRange(CurrentResults)
            End While
            Return Results
        End Function
        <System.Runtime.CompilerServices.Extension> _
        Public Sub RelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, Entities As IList(Of Entity))
            Service.RelateEntities(Target, IntermediateEntityName, Entities.ToArray())
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub RelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, ParamArray Entities As Entity())
            Dim References As IList(Of EntityReference) = Entities.ToList()
            Service.RelateEntities(Target, IntermediateEntityName, References)
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub RelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, Entities As IList(Of EntityReference))
            Service.RelateEntities(Target, IntermediateEntityName, Entities.ToArray())
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub RelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, ParamArray Entities As EntityReference())
            Dim Request As New AssociateRequest() With { _
                .Relationship = New Relationship(IntermediateEntityName), _
                .Target = Target _
            }
            Request.RelatedEntities = New EntityReferenceCollection(Entities)
            Service.Execute(Request)
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub UnRelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, Entities As IList(Of Entity))
            Service.UnRelateEntities(Target, IntermediateEntityName, Entities.ToArray())
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub UnRelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, ParamArray Entities As Entity())
            Dim References As IList(Of EntityReference) = Entities.ToList()
            Service.UnRelateEntities(Target, IntermediateEntityName, References)
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub UnRelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, Entities As IList(Of EntityReference))
            Service.UnRelateEntities(Target, IntermediateEntityName, Entities.ToArray())
        End Sub
        <System.Runtime.CompilerServices.Extension> _
        Public Sub UnRelateEntities(Service As IOrganizationService, Target As EntityReference, IntermediateEntityName As String, ParamArray Entities As EntityReference())
            Dim Request As New DisassociateRequest() With { _
                .Target = Target, _
                .Relationship = New Relationship(IntermediateEntityName) _
            }
            Request.RelatedEntities = New EntityReferenceCollection(Entities)
            Service.Execute(Request)
        End Sub
    End Module
End Namespace